program htm2clip;

// htm2clip by A.Ruzanov
// 15.02.2009

uses
  windows,
  messages,
  KOL;

var
  str, txt, path: string;
  time: dword;

function FormatHTMLClipboardHeader(HTMLText: string): string;
const
 CrLf = #13#10;
begin
 Result := 'Version:1.0' + CrLf;
 Result := Result + 'StartHTML:<<<<<<<<1' + CrLf;
 Result := Result + 'EndHTML:<<<<<<<<2' + CrLf;
 Result := Result + 'StartFragment:<<<<<<<<3' + CrLf;
 Result := Result + 'EndFragment:<<<<<<<<4' + CrLf;
 Result := Result + HTMLText + CrLf;
 StrReplace(Result, '<<<<<<<<1', Format('%.9d', [Pos(CrLf+CrLf ,Result)+3]));
 StrReplace(Result, '<<<<<<<<2', Format('%.9d', [Length(Result)]));
 StrReplace(Result, '<<<<<<<<3', Format('%.9d', [Pos('<!--StartFragment-->', Result)+19]));
 StrReplace(Result, '<<<<<<<<4', Format('%.9d', [Pos('<!--EndFragment-->', Result)-1]));
end;

procedure AddToClipBoard(const str, cf_format: string);
var gbl: THandle;
    st: PChar;
    cf: UINT;
begin
  if OpenClipboard(0) and (str <> '') then
  begin
    cf := RegisterClipboardFormat(PChar(cf_format));
    gbl := GlobalAlloc(GMEM_DDESHARE or GMEM_MOVEABLE, Length(str) + 1);
    if gbl <> 0 then
      begin
      st := GlobalLock( gbl );
      Move( str[ 1 ], st^, Length(str) + 1);
      GlobalUnlock( gbl );
      SetClipboardData( cf, gbl );
      end;
    CloseClipboard;
  end;
end;

procedure ConvertHtml2Unicode;
var gbl: THandle;
    str: PChar;
    wstr: WideString;
    CF_HTML: UINT;
begin
  if OpenClipboard(0) then
  begin
    CF_HTML := RegisterClipboardFormat('HTML Format');
    if IsClipboardFormatAvailable(CF_HTML) And Not IsClipboardFormatAvailable(CF_GDIOBJFIRST) then
      begin
        gbl := GetClipboardData(CF_HTML);
        if gbl <> 0 then
        begin
          str := GlobalLock(gbl);
          if str <> nil then
          begin
            wstr := Utf8Decode(CopyEnd(str, Pos('<', str)));
            GlobalUnlock(gbl);
            gbl := GlobalAlloc( GMEM_DDESHARE or GMEM_MOVEABLE, (Length( wstr ) + 1) * 2 );
              if gbl <> 0 then
              begin
                str := GlobalLock( gbl );
                Move( wstr[ 1 ], str^, (Length( wstr ) + 1) * 2 );
                GlobalUnlock( gbl );
                SetClipboardData( CF_GDIOBJFIRST, gbl );
              end;
          end;
        end;
      end;
    CloseClipboard;
  end;
end;

function htm2rtf(const Source: string): String;
type
  TConvertSettings = record
  PreserveTables : integer;             //1 - preserve tables, 0 - transfer to text
  PreserveImages : integer;             //1 - preserve images,  0 - skip
  PreserveHyperlinks : integer;         //1 - preserve hyperlinks, 0 - skip
  PreserveFontFace : integer;           //1 - preserve font faces as in HTML, 0 - all font faces will be as in 'FontFace'
  PreserveFontSize : integer;           //1 - preserve font sizes as in HTML, 0 - all font sizes will be as in 'FontSize'
  PreserveFontColor : integer;          //1 - preserve font colors as in HTML, 0 - black font color
  PreserveBackgroundColor : integer;    //1 - preserve bacground colors as in HTML, 0 - skip
  PreserveAlignment : integer;          //1 - preserve alignment as in HTML, 0 - all text will have align as in 'PageAlignment'
  PreserveTableWidth : integer;         //1 - preserve width of columns
  PreserveNestedTables : integer;       //1 - preserve nested tables, 0 - translate nested tables to plain tables
  PageMarginLeft : integer;             //page margin left, mm. For example 10
  PageMarginRight : integer;            //page margin right, mm. For example 10
  PageMarginTop : integer;              //page margin top, mm. For example 10
  PageMarginBottom : integer;           //page margin bottom, mm. For example 10
  BorderVisibility : integer;           //table borders: 0 - visible borders, 1 - hidden borders, 2 - as in HTML
  PageOrientation : integer;            //page orientation: 0 - Portrait, 1 - Landscape
  PageSize : integer;                   //page size: 0 - A4, 1 - A3, 2 - A5, 3 - B5, 4 - Letter, 5 - Legal, 6 - Executive, 7 - Monarh
  FontFace : integer;                   //default font face: Arial - 0, Times New Roman - 1, Verdana - 2, Helvetica - 3, Courier - 4, Courier New - 5, Times - 6, Georgia - 7, MS Sans Serif - 8,
		                                    //Futura - 9, Arial Narrow - 10, Garamond - 11, Impact - 12, Lucida Console - 13, Tahoma - 14, Inform - 15, Symbol - 16, WingDings - 17, Traditional Arabic - 18
  FontSize : integer;                   //default font size, any value from 6 to 72
  PageAlignment : integer;              //page alignment: 0 - left, 1 - center, 2 - right, 3 - justify
  RtfLanguage : integer;                //RTF language: English - 1033, Albanian - 1052, Belgian - 2067, Bulgarian - 1026, Hungarian - 1038, Danish - 1030, Spanish - 3082, Latvian - 1062, Lithuanian - 1063,
                                        //German - 1031, Netherlands - 1043, Norwegian - 2068, Portuguese - 2070, Romanian - 1048, Russian - 1049, Ukrainian - 1058, Finnish - 1035, French - 1036,
                                        //Czech - 1029, Swedish - 1053, Arabic - 1053, Turkish - 1055, Japanese - 932, SimplifiedChinese - 936, TraditionalChinese - 950, Korean - 949, Thai = 874
  Encoding : integer;                   //AutoSelect - 0, ISO-8859-1 - 1, ISO-8859-5 - 2, KOI8-R - 3, Windows-1251 - 4, UTF-8 - 5, Windows-1254 - 6, Windows-1256 - 7,
                        		            //Windows-1250 - 8, Windows-1252 - 9, Windows-1253 - 10, Windows-1255 - 11, Windows-1257 - 12, Windows-1258 - 13
  OutputTextFormat : integer;           //Output Format: Rtf - 0, Text - 1, Doc - 2 (only file with .doc extension)  
  PreservePageBreaks : integer;         //1 - preserve page-breaks
  ImageCompatible  : integer;           //type of produced images: Word - 0, WordPad - 1
  PageNumbers : integer;                //page numbers: 0 - disable, 1 - numbers from first page, 2 - from second page
  PageHeader : Array [0..149] of Char;  //page header, any string
  PageFooter : Array [0..149] of Char;  //page footer, any string
  HtmlPath : Array [0..649] of Char;    //html path for method 'htmltortf_string', will be used for find images (now is not used)
  PageNumbersAlignV : integer;          //page numbers vertical align: Top - 4, Bottom - 5
  PageNumbersAlignH : integer;          //page numbers horizontal align: 0 - left, 1 - center, 2 - right, 3 - justify
  PreserveHR : integer;                 //1 - preseve <hr>, 0 - skip
end;

var
  FConvertSettings: TConvertSettings;
  lOutString: pchar;
  LibHandle: THandle;
  htmltortf_string: function (ARTF: string; AHTML: string; AOptions: TConvertSettings): pchar; cdecl;
  flushlist: function: integer; cdecl;

begin
    lOutString := nil;
    LibHandle := LoadLibrary('htmltortf.dll');
    if LibHandle <> 0 then
    begin
    @htmltortf_string := GetProcAddress(LibHandle, 'htmltortf_string');
    @flushlist := GetProcAddress(LibHandle, 'flushlist');
    If (@htmltortf_string <> nil) And (@flushlist <> nil) Then
    begin
    FConvertSettings.PreserveTables:=1;
    FConvertSettings.PreserveImages:=1;
    FConvertSettings.PreserveHyperlinks:=1;
    FConvertSettings.PreserveFontFace:=1;
    FConvertSettings.PreserveFontSize:=1;
    FConvertSettings.PreserveFontColor:=1;
    FConvertSettings.PreserveBackgroundColor:=1;
    FConvertSettings.PreserveAlignment:=1;
    FConvertSettings.PreserveTableWidth:=1;
    FConvertSettings.PreserveNestedTables:=1;
    FConvertSettings.PageMarginLeft:=20;
    FConvertSettings.PageMarginRight:=25;
    FConvertSettings.PageMarginTop:=20;
    FConvertSettings.PageMarginBottom:=20;
    FConvertSettings.BorderVisibility:=1;
    FConvertSettings.PageOrientation:=0;
    FConvertSettings.PageSize:=0;
    FConvertSettings.FontFace:=0;
    FConvertSettings.FontSize:=10;
    FConvertSettings.PageAlignment:=0;
    FConvertSettings.RtfLanguage:=1033;
    FConvertSettings.Encoding:=5;
    FConvertSettings.OutputTextFormat:=0;
    FConvertSettings.PreservePageBreaks:=1;
    FConvertSettings.ImageCompatible:=0;
    FConvertSettings.PageNumbers:=0;
    FConvertSettings.PageHeader:='';
    FConvertSettings.PageFooter:='';
    FConvertSettings.HtmlPath:='';
    FConvertSettings.PageNumbersAlignV:=5;
    FConvertSettings.PageNumbersAlignH:=2;
    FConvertSettings.PreserveHR:=1;
    Result := htmltortf_string(pchar(Source), lOutString, FConvertSettings);
    flushlist();
    end;
    FreeLibrary(LibHandle);
    end;
end;

begin
If ParamCount = 0 Then
  begin
  MessageBox(0, PChar('htm2clip for Opera 9'), 'About', 64);
  Exit;
  end;

If (ParamCount = 1) Then
 If ParamStr(1) = '/paste' Then
  begin
  ConvertHtml2Unicode;
  keybd_event(VK_SHIFT, 0, 0, 0);
  keybd_event(VK_INSERT	, 0, 0, 0);
  keybd_event(VK_INSERT	, 0, KEYEVENTF_KEYUP, 0);
  keybd_event(VK_SHIFT, 0, KEYEVENTF_KEYUP, 0);
  end
 Else
  begin
  txt := Clipboard2Text;
  time := GetTickCount + 1000;
  keybd_event(VK_CONTROL, 0, 0, 0);
  keybd_event(VK_INSERT	, 0, 0, 0);
  keybd_event(VK_INSERT	, 0, KEYEVENTF_KEYUP, 0);
  keybd_event(VK_CONTROL, 0, KEYEVENTF_KEYUP, 0);

  while (txt = Clipboard2Text) And (GetTickCount < time) do sleep(100);
  path := ParamStr(1);
  If FileExists(path) Then
	begin
	str := ExcludeTrailingChar(StrLoadFromFile(path), #00 );
	If Pos('SourceURL:', str)= 1 Then
	  begin
	  AddToClipBoard(FormatHTMLClipboardHeader(str), 'HTML Format');
	  If FileExists(GetStartDir+'htmltortf.dll') Then AddToClipBoard(htm2rtf(str), 'Rich Text Format');
	  DeleteFile(PChar(path));
	  end;
	str := '';
	end;
  end;
end.



